==> certillion-client-library-1.1.7-sources.jar: br/com/esec/icpm/libs/signature/response/handler/simple/SignatureSimpleAsynchHandler.java <==
package br.com.esec.icpm.libs.signature.response.handler.simple;

import java.io.IOException;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.common.util.concurrent.FutureCallback;
import com.google.common.util.concurrent.Futures;
import com.google.common.util.concurrent.ListenableFuture;

import br.com.esec.icpm.libs.Server;
import br.com.esec.icpm.libs.signature.response.ThreadPool;
import br.com.esec.icpm.libs.signature.response.handler.SignatureAsynchHandler;
import br.com.esec.icpm.libs.signature.response.polling.SimplePollingService;
import br.com.esec.icpm.mss.ws.SignatureRespType;
import br.com.esec.icpm.mss.ws.SignatureStatusRespType;
import br.com.esec.icpm.server.ws.ICPMException;

public class SignatureSimpleAsynchHandler extends BaseSimpleSignatureHandler implements SignatureAsynchHandler {

	private static Logger log = LoggerFactory.getLogger(SignatureSimpleAsynchHandler.class);
	
	public SignatureSimpleAsynchHandler(Server server, long transactionId) {
		super(server, transactionId);
	}

	public SignatureSimpleAsynchHandler(Server server, SignatureRespType response) {
		super(server, response);
	}

	@Override
	public SignatureSimpleAsynchHandler waitTo() throws ICPMException, InterruptedException, TimeoutException {
		return waitTo(DEFAULT_TIMEOUT);
	}

	@Override
	public SignatureSimpleAsynchHandler waitTo(long timeout) throws ICPMException, InterruptedException, TimeoutException {
		Future<SignatureStatusRespType> result = SimplePollingService.getInstance().status(server, transactionId);
		try {
			SignatureStatusRespType signatureStatusResp = result.get(timeout, TimeUnit.SECONDS);
			
			log.info("Simple signature arrived.");
			
			parseStatus(signatureStatusResp);
		} catch (TimeoutException e) {
			result.cancel(true); // FIXME: Não continua a transação de quem chamou. Pq?
			throw e;
		} catch (ExecutionException e) {
			throw new IllegalStateException(e);
		} catch (IOException e) {
			throw new IllegalStateException(e);
		}

		return this;
	}

	@Override
	public SignatureAsynchHandler addListener(final Listener listener) throws ICPMException, InterruptedException, TimeoutException {
		final ListenableFuture<SignatureStatusRespType> result = SimplePollingService.getInstance().status(server, transactionId);
		Futures.addCallback(result, new FutureCallback<SignatureStatusRespType>() {
			@Override
			public void onSuccess(SignatureStatusRespType signatureStatus) {
				try {
					log.info("Simple signature arrived.");
					SignatureSimpleAsynchHandler.this.parseStatus(signatureStatus);
					listener.onSuccess(SignatureSimpleAsynchHandler.this);
				} catch (IOException e) {
					throw new IllegalStateException(e);
				}
			}

			@Override
			public void onFailure(Throwable t) {
				listener.onFailed(t);
				
			}
		}, ThreadPool.instance);
		
		return this;
	}
	
	private void parseStatus(SignatureStatusRespType signatureStatusResp) throws IOException {
		if (signatureStatusResp.getStatus() != null) {
			statusCode = signatureStatusResp.getStatus().getStatusCode();
			statusMessage = signatureStatusResp.getStatus().getStatusMessage();
			signature = parseSignature(statusCode, signatureStatusResp.getSignature());
		}
	}
	
}

==> certillion-client-library-1.1.12-sources.jar: br/com/esec/icpm/libs/signature/response/handler/simple/SignatureSimpleAsynchHandler.java <==
package br.com.esec.icpm.libs.signature.response.handler.simple;

import java.io.IOException;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.common.util.concurrent.FutureCallback;
import com.google.common.util.concurrent.Futures;
import com.google.common.util.concurrent.ListenableFuture;

import br.com.esec.icpm.libs.Server;
import br.com.esec.icpm.libs.signature.response.CertillionThreadPool;
import br.com.esec.icpm.libs.signature.response.handler.SignatureAsynchHandler;
import br.com.esec.icpm.libs.signature.response.polling.SimplePollingService;
import br.com.esec.icpm.mss.ws.SignatureRespType;
import br.com.esec.icpm.mss.ws.SignatureStatusRespType;
import br.com.esec.icpm.server.ws.ICPMException;

/**
 * Response Simple Asynchronous Signature Handler.
 * 
 * @author Tales Porto (tporto@esec.com.br|talesap@gmail.com)
 */
public class SignatureSimpleAsynchHandler extends BaseSimpleSignatureHandler implements SignatureAsynchHandler {

	private static Logger log = LoggerFactory.getLogger(SignatureSimpleAsynchHandler.class);
	
	public SignatureSimpleAsynchHandler(Server server, long transactionId) {
		super(server, transactionId);
	}

	public SignatureSimpleAsynchHandler(Server server, SignatureRespType response) {
		super(server, response);
	}

	@Override
	public SignatureSimpleAsynchHandler waitTo() throws ICPMException, InterruptedException, TimeoutException {
		return waitTo(DEFAULT_TIMEOUT);
	}

	@Override
	public SignatureSimpleAsynchHandler waitTo(long timeout) throws ICPMException, InterruptedException, TimeoutException {
		Future<SignatureStatusRespType> result = SimplePollingService.getInstance().status(server, transactionId);
		try {
			SignatureStatusRespType signatureStatusResp = result.get(timeout, TimeUnit.SECONDS);
			
			log.info("Simple signature arrived.");
			
			parseStatus(signatureStatusResp);
		} catch (TimeoutException e) {
			result.cancel(true); // FIXME: Não continua a transação de quem chamou. Pq?
			throw e;
		} catch (ExecutionException e) {
			throw new IllegalStateException(e);
		} catch (IOException e) {
			throw new IllegalStateException(e);
		}

		return this;
	}

	@Override
	public SignatureAsynchHandler addListener(final Listener listener) throws ICPMException, InterruptedException, TimeoutException {
		final ListenableFuture<SignatureStatusRespType> result = SimplePollingService.getInstance().status(server, transactionId);
		Futures.addCallback(result, new FutureCallback<SignatureStatusRespType>() {
			@Override
			public void onSuccess(SignatureStatusRespType signatureStatus) {
				try {
					log.info("Simple signature arrived.");
					SignatureSimpleAsynchHandler.this.parseStatus(signatureStatus);
					listener.onSuccess(SignatureSimpleAsynchHandler.this);
				} catch (IOException e) {
					throw new IllegalStateException(e);
				}
			}

			@Override
			public void onFailure(Throwable t) {
				listener.onFailed(t);
				
			}
		}, CertillionThreadPool.getInstance());
		
		return this;
	}
	
	private void parseStatus(SignatureStatusRespType signatureStatusResp) throws IOException {
		if (signatureStatusResp.getStatus() != null) {
			statusCode = signatureStatusResp.getStatus().getStatusCode();
			statusMessage = signatureStatusResp.getStatus().getStatusMessage();
			signature = parseSignature(statusCode, signatureStatusResp.getSignature());
		}
	}
	
}
